==> esbuild-plugin-scriptable-0.2.0.tgz: package/src/scriptable-banner.ts <==
import type {ScriptableManifest} from '@scriptables/manifest';
import {generateScriptableBanner} from '@scriptables/manifest/generateScriptableBanner';
import type {Plugin} from 'esbuild';
import {existsSync} from 'fs';
import {readFile, writeFile} from 'fs/promises';
import {basename, dirname, posix, resolve, sep} from 'path';

const DEFAULT_MANIFEST_EXTENSIONS = ['.manifest.json', '.manifest', '.json'];

export interface ScriptablePluginOptions {
  /** Custom manifest file extensions */
  manifestExtensions?: string[];
  /** Show warning when manifest file is missing */
  warnOnMissingManifest?: boolean;
  /** Show warning when manifest file is invalid */
  warnOnInvalidManifest?: boolean;
  /** Show warning when no matching entry point is found */
  warnOnMissingEntry?: boolean;
  /** Custom manifest file path mapping function */
  getManifestPath?: (entryPath: string) => string;
}

export function scriptableBanner(options: ScriptablePluginOptions = {}): Plugin {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
  } = options;

  return {
    name: 'scriptable',
    setup(build) {
      let manifestCache = new Map<string, ScriptableManifest>();

      // Ensure metafile is enabled
      if (!build.initialOptions.metafile) {
        console.warn(
          'Warning: "metafile" option was not enabled. ' +
            'Automatically setting "metafile: true" to allow scriptable plugin to modify output files.',
        );
        build.initialOptions.metafile = true;
      }

      build.onStart(() => {
        manifestCache.clear();
      });

      build.onResolve({filter: /\.[tj]sx?$/}, async args => {
        if (
          Array.isArray(build.initialOptions.entryPoints) &&
          build.initialOptions.entryPoints.map(String).includes(args.path)
        ) {
          const dir = dirname(args.path);
          const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, '');

          const manifestPath = getManifestPath
            ? getManifestPath(args.path)
            : manifestExtensions.map(ext => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);

          if (manifestPath) {
            try {
              const manifestContent = await readFile(manifestPath, 'utf8');
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = posix.normalize(args.path.split(sep).join('/'));

              if (!manifest.name || !manifest.author || !manifest.version) {
                if (warnOnInvalidManifest) {
                  console.warn(`Invalid manifest file: ${manifestPath} - missing required fields`);
                }
                return null;
              }

              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              console.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            console.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });

      build.onEnd(async result => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          console.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. ' +
              'Ensure "metafile: true" is set in esbuild options if write is enabled.',
          );
          return;
        }

        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          if (build.initialOptions.write !== false && !existsSync(outputPath)) {
            console.warn(`Output file does not exist: ${outputPath}`);
            continue;
          }

          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? '';
          const normalizedEntryPath = entryPoint ? posix.normalize(entryPoint.split(sep).join('/')) : '';

          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              console.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }

          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner(manifest);

            if (result.outputFiles) {
              // Modify in-memory output files (write: false)
              const outputFile = result.outputFiles.find(file => file.path === outputPath || file.path === '<stdout>');
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              // Modify written output files (write: true)
              const content = await readFile(outputPath, 'utf-8');
              const updatedContent = banner + content;
              await writeFile(outputPath, updatedContent, 'utf8');
            }
          } catch (error) {
            console.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    },
  };
}

==> esbuild-plugin-scriptable-0.3.0.tgz: package/src/scriptable-banner.ts <==
import type {ScriptableManifest} from '@scriptables/manifest';
import {generateScriptableBanner} from '@scriptables/manifest/generateScriptableBanner';
import type {Plugin} from 'esbuild';
import {existsSync} from 'fs';
import {readFile, writeFile} from 'fs/promises';
import micromatch from 'micromatch';
import {basename, dirname, posix, resolve, sep} from 'path';

import {createLogger} from './logger';
import {createPatternMatcher} from './utils';

const DEFAULT_MANIFEST_EXTENSIONS = ['.manifest.json', '.manifest', '.json'];

export interface ScriptablePluginOptions {
  /** Custom manifest file extensions */
  manifestExtensions?: string[];
  /** Show warning when manifest file is missing */
  warnOnMissingManifest?: boolean;
  /** Show warning when manifest file is invalid */
  warnOnInvalidManifest?: boolean;
  /** Show warning when no matching entry point is found */
  warnOnMissingEntry?: boolean;
  /** Custom manifest file path mapping function */
  getManifestPath?: (entryPath: string) => string;
  /**glob matching pattern, supports ! prefix to exclude files */
  patterns?: string | string[];
}

export function scriptableBanner(options: ScriptablePluginOptions = {}): Plugin {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
    patterns = ['**/*.{ts,tsx,js,jsx}'],
  } = options;

  const logger = createLogger('scriptable-banner');

  return {
    name: 'scriptable',
    setup(build) {
      let manifestCache = new Map<string, ScriptableManifest>();

      // Ensure metafile is enabled
      if (!build.initialOptions.metafile) {
        logger.warn(
          'Warning: "metafile" option was not enabled. ' +
            'Automatically setting "metafile: true" to allow scriptable plugin to modify output files.',
        );
        build.initialOptions.metafile = true;
      }

      build.onStart(() => {
        manifestCache.clear();
      });

      build.onResolve({filter: /.*/}, async args => {
        if (!micromatch.isMatch(args.path, patterns)) {
          return null;
        }

        if (
          Array.isArray(build.initialOptions.entryPoints) &&
          build.initialOptions.entryPoints.map(String).includes(args.path)
        ) {
          const dir = dirname(args.path);
          const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, '');

          const manifestPath = getManifestPath
            ? getManifestPath(args.path)
            : manifestExtensions.map(ext => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);

          if (manifestPath) {
            try {
              const manifestContent = await readFile(manifestPath, 'utf8');
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = posix.normalize(args.path.split(sep).join('/'));

              if (!manifest.name || !manifest.author || !manifest.version) {
                if (warnOnInvalidManifest) {
                  logger.warn(`Invalid manifest file: ${manifestPath} - missing required fields`);
                }
                return null;
              }

              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            logger.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });

      build.onEnd(async result => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. ' +
              'Ensure "metafile: true" is set in esbuild options if write is enabled.',
          );
          return;
        }

        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          if (build.initialOptions.write !== false && !existsSync(outputPath)) {
            logger.warn(`Output file does not exist: ${outputPath}`);
            continue;
          }

          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? '';
          const normalizedEntryPath = entryPoint ? posix.normalize(entryPoint.split(sep).join('/')) : '';

          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }

          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner(manifest);

            if (result.outputFiles) {
              const outputFile = result.outputFiles.find(file => file.path === outputPath || file.path === '<stdout>');
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              const content = await readFile(outputPath, 'utf-8');
              const updatedContent = banner + content;
              await writeFile(outputPath, updatedContent, 'utf8');
            }
          } catch (error) {
            logger.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    },
  };
}
